/**
 * @popperjs/core v2.0.0-next.11
 */

var getBoundingClientRect = (function (element) {
  var rect = JSON.parse(JSON.stringify(element.getBoundingClientRect()));
  rect.x = rect.left;
  rect.y = rect.top;
  return rect;
});

var getNodeName = (function (element) {
  return element ? (element.nodeName || '').toUpperCase() : null;
});

var getParentNode = (function (element) {
  if (getNodeName(element) === 'HTML') {
    // DocumentElement detectedF
    return element;
  }

  return element.parentNode || // DOM Element detected
  // $FlowFixMe: need a better way to handle this...
  element.host || // ShadowRoot detected
  document.ownerDocument || // Fallback to ownerDocument if available
  document.documentElement // Or to documentElement if everything else fails
  ;
});

function getWindow(node) {
  var ownerDocument = node != null && typeof node === 'object' && node.hasOwnProperty('ownerDocument') ? // $FlowFixMe: assume ownerDocument to be the one we are looking for
  node.ownerDocument : null;
  return ownerDocument != null ? ownerDocument.defaultView : window;
}

function getComputedStyle(element) {
  return getWindow(element).getComputedStyle(element);
}

function getScrollParent(node) {
  if (!node) {
    return document.body;
  }

  if (['HTML', 'BODY', '#document'].includes(getNodeName(node))) {
    return node.ownerDocument.body;
  }

  if (node instanceof getWindow(node).HTMLElement) {
    // Firefox want us to check `-x` and `-y` variations as well
    var _getComputedStyle = getComputedStyle(node),
        overflow = _getComputedStyle.overflow,
        overflowX = _getComputedStyle.overflowX,
        overflowY = _getComputedStyle.overflowY;

    if (/(auto|scroll|overlay)/.test(overflow + overflowY + overflowX)) {
      return node;
    }
  }

  return getScrollParent(getParentNode(node));
}

function listScrollParents(element, list) {
  if (list === void 0) {
    list = [];
  }

  var scrollParent = getScrollParent(element);
  var isBody = getNodeName(scrollParent) === 'BODY';
  var target = isBody ? getWindow(scrollParent) : scrollParent;
  var updatedList = list.concat(target);
  return isBody ? updatedList : updatedList.concat(listScrollParents(getParentNode(target)));
}

function getWindowScroll(node) {
  var win = getWindow(node);
  var scrollLeft = win.pageXOffset;
  var scrollTop = win.pageYOffset;
  return {
    scrollLeft: scrollLeft,
    scrollTop: scrollTop
  };
}

function getHTMLElementScroll(element) {
  return {
    scrollLeft: element.scrollLeft,
    scrollTop: element.scrollTop
  };
}

function getElementScroll(node) {
  if (node === getWindow(node) || !(node instanceof getWindow(node).HTMLElement)) {
    return getWindowScroll(node);
  } else {
    return getHTMLElementScroll(node);
  }
}

var getScrollSum = (function (scrollParents) {
  return scrollParents.reduce(function (scroll, scrollParent) {
    var nodeScroll = getElementScroll(scrollParent);
    scroll.scrollTop += nodeScroll.scrollTop;
    scroll.scrollLeft += nodeScroll.scrollLeft;
    return scroll;
  }, {
    scrollTop: 0,
    scrollLeft: 0
  });
});

function getOffsetParent(element) {
  var offsetParent = element instanceof getWindow(element).HTMLElement ? element.offsetParent : null;
  var window = getWindow(element);

  if (getNodeName(offsetParent) === 'BODY') {
    return window;
  }

  return offsetParent || window;
}

var unwrapVirtualElement = (function (element) {
  return element instanceof getWindow(element).Element ? element : // $FlowFixMe: type refinement not working
  element.contextElement;
});

// offsetParent

var getElementClientRect = (function (element, isFixed) {
  if (isFixed === void 0) {
    isFixed = false;
  }

  var unwrappedElement = unwrapVirtualElement(element);
  var win = getWindow(unwrappedElement);
  var rect = getBoundingClientRect(element);
  var scrollParentsScrollSum = getScrollSum(listScrollParents(unwrappedElement));
  var directOffsetParent = getOffsetParent(unwrappedElement);
  var directOffsetParentRect = directOffsetParent instanceof win.Element && !isFixed ? getBoundingClientRect(directOffsetParent) : {
    left: 0,
    top: 0
  };
  var ancestorOffsetParents = isFixed ? [directOffsetParent] : [];
  var currentOffsetParent = directOffsetParent;

  while (currentOffsetParent instanceof win.Element) {
    currentOffsetParent = getOffsetParent(currentOffsetParent);
    ancestorOffsetParents.push(currentOffsetParent);
  }

  var ancestorOffsetParentScrollSum = getScrollSum(ancestorOffsetParents);
  var width = rect.width;
  var height = rect.height;
  var x = rect.left + scrollParentsScrollSum.scrollLeft - ancestorOffsetParentScrollSum.scrollLeft - directOffsetParentRect.left;
  var y = rect.top + scrollParentsScrollSum.scrollTop - ancestorOffsetParentScrollSum.scrollTop - directOffsetParentRect.top;
  return {
    width: width,
    height: height,
    x: x,
    y: y
  };
});

var getElementMargins = (function (element) {
  // get the element margins, we need them to properly align the popper
  var styles = getComputedStyle(element);
  var top = parseFloat(styles.marginTop) || 0;
  var right = parseFloat(styles.marginRight) || 0;
  var bottom = parseFloat(styles.marginBottom) || 0;
  var left = parseFloat(styles.marginLeft) || 0;
  return {
    top: top,
    right: right,
    bottom: bottom,
    left: left
  };
});

var addClientRectMargins = (function (rect, element) {
  var margins = getElementMargins(element);
  return {
    width: rect.width + margins.left + margins.right,
    height: rect.height + margins.top + margins.bottom,
    y: rect.y - margins.top,
    x: rect.x - margins.left
  };
});

var unwrapJqueryElement = (function (element) {
  return (// $FlowFixMe: need to get type refinement work
    element.hasOwnProperty('jquery') ? element[0] : element
  );
});

// source: https://stackoverflow.com/questions/49875255
var order = function order(modifiers) {
  // indexed by name
  var mapped = modifiers.reduce(function (mem, i) {
    mem[i.name] = i;
    return mem;
  }, {}); // inherit all dependencies for a given name

  var inherited = function inherited(i) {
    return mapped[i] && mapped[i].requires ? mapped[i].requires.reduce(function (mem, i) {
      return [].concat(mem, [i], inherited(i));
    }, []) : [];
  }; // order ...


  var ordered = modifiers.sort(function (a, b) {
    return !!~inherited(b.name).indexOf(a.name) ? -1 : 1;
  });
  return ordered;
};

var orderModifiers = (function (modifiers) {
  return ['read', 'main', 'afterMain', 'write'].reduce(function (acc, currentPhase) {
    return acc.concat(order(modifiers.filter(function (modifier) {
      return modifier.phase === currentPhase;
    })));
  }, []);
});

function expandToHashMap(value, keys) {
  return keys.reduce(function (hashMap, key) {
    hashMap[key] = value;
    return hashMap;
  }, {});
}

function microtaskDebounce(fn) {
  var pending;
  return function () {
    if (!pending) {
      pending = new Promise(function (resolve) {
        Promise.resolve().then(function () {
          pending = undefined;
          resolve(fn());
        });
      });
    }

    return pending;
  };
}

var format = (function (str) {
  for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  return [].concat(args).reduce(function (p, c) {
    return p.replace(/%s/, c);
  }, str);
});

var top = 'top';
var bottom = 'bottom';
var right = 'right';
var left = 'left';
var basePlacements = [top, bottom, right, left];
var start = 'start';
var end = 'end';
var placements = basePlacements.reduce(function (acc, placement) {
  return acc.concat([placement, placement + "-" + start, placement + "-" + end]);
}, []); // modifiers that need to read the DOM

var read = 'read'; // pure-logic modifiers

var main = 'main'; // pure-logic modifiers that run after the main phase (such as computeStyles)

var write = 'write';

var ERROR_MESSAGE = 'Popper: modifier "%s" provided an invalid %s property, expected %s but got %s';
var VALID_PROPERTIES = ['name', 'enabled', 'phase', 'fn', 'onLoad', 'requires', 'options'];
var validateModifiers = (function (modifiers) {
  modifiers.forEach(function (modifier) {
    Object.keys(modifier).forEach(function (key) {
      switch (key) {
        case 'name':
          if (typeof modifier.name !== 'string') {
            console.error(format(ERROR_MESSAGE, String(modifier.name), '"name"', '"string"', "\"" + String(modifier.name) + "\""));
          }

          break;

        case 'enabled':
          if (typeof modifier.enabled !== 'boolean') {
            console.error(format(ERROR_MESSAGE, modifier.name, '"enabled"', '"boolean"', "\"" + String(modifier.enabled) + "\""));
          }

        case 'phase':
          if (![read, main, write].includes(modifier.phase)) {
            console.error(format(ERROR_MESSAGE, modifier.name, '"phase"', 'either "read", "main" or "write"', "\"" + String(modifier.phase) + "\""));
          }

          break;

        case 'fn':
          if (typeof modifier.fn !== 'function') {
            console.error(format(ERROR_MESSAGE, modifier.name, '"fn"', '"function"', "\"" + String(modifier.fn) + "\""));
          }

          break;

        case 'onLoad':
          if (typeof modifier.onLoad !== 'function') {
            console.error(format(ERROR_MESSAGE, modifier.name, '"onLoad"', '"function"', "\"" + String(modifier.fn) + "\""));
          }

          break;

        case 'onDestroy':
          if (typeof modifier.onLoad !== 'function') {
            console.error(format(ERROR_MESSAGE, modifier.name, '"onDestroy"', '"function"', "\"" + String(modifier.fn) + "\""));
          }

          break;

        case 'requires':
          if (!Array.isArray(modifier.requires)) {
            console.error(format(ERROR_MESSAGE, modifier.name, '"requires"', '"array"', "\"" + String(modifier.requires) + "\""));
          }

          break;

        case 'options':
        case 'data':
          break;

        default:
          console.error("PopperJS: an invalid property has been provided to the \"" + modifier.name + "\" modifier, valid properties are " + VALID_PROPERTIES.map(function (s) {
            return "\"" + s + "\"";
          }).join(', ') + "; but \"" + key + "\" was provided.");
      }
    });
  });
});

// DOM Utils
var INVALID_ELEMENT_ERROR = 'Popper: Invalid reference or popper argument provided to Popper, they must be either a valid DOM element, virtual element, or a jQuery-wrapped DOM element.';
var INFINITE_LOOP_ERROR = 'Popper: An infinite loop in the modifiers cycle has been detected! The cycle has been interrupted to prevent a browser crash.';

var areValidElements = function areValidElements() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return !args.some(function (element) {
    return !(element && typeof element.getBoundingClientRect === 'function');
  });
};

var defaultOptionsValue = {
  placement: 'bottom',
  eventListeners: {
    scroll: true,
    resize: true
  },
  modifiers: [],
  strategy: 'absolute'
};
function popperGenerator(generatorOptions) {
  if (generatorOptions === void 0) {
    generatorOptions = {};
  }

  var _generatorOptions = generatorOptions,
      _generatorOptions$def = _generatorOptions.defaultModifiers,
      defaultModifiers = _generatorOptions$def === void 0 ? [] : _generatorOptions$def,
      _generatorOptions$def2 = _generatorOptions.defaultOptions,
      defaultOptions = _generatorOptions$def2 === void 0 ? defaultOptionsValue : _generatorOptions$def2;
  return function createPopper(reference, popper, options) {
    if (options === void 0) {
      options = defaultOptions;
    }

    var instance = {};
    var state = {
      placement: 'bottom',
      orderedModifiers: [],
      options: defaultOptions,
      modifiersData: {}
    }; // Syncronous and forcefully executed update
    // it will always be executed even if not necessary, usually NOT needed
    // use Popper#update instead

    instance.forceUpdate = function () {
      var _state$elements = state.elements,
          referenceElement = _state$elements.reference,
          popperElement = _state$elements.popper; // Don't proceed if `reference` or `popper` are not valid elements anymore

      if (!areValidElements(referenceElement, popperElement)) {
        if (process.env.NODE_ENV !== "production") {
          console.error(INVALID_ELEMENT_ERROR);
        }

        return;
      }

      var isFixed = state.options.strategy === 'fixed'; // Get initial measurements
      // these are going to be used to compute the initial popper offsets
      // and as cache for any modifier that needs them later

      state.measures = {
        reference: getElementClientRect(referenceElement, isFixed),
        // CSS marginsc an be applied to popper elements to quickly
        // apply offsets dynamically based on some CSS selectors.
        // For this reason we include margins in this calculation.
        popper: addClientRectMargins(getElementClientRect(popperElement, isFixed), popperElement)
      }; // Modifiers have the ability to read the current Popper state, included
      // the popper offsets, and modify it to address specifc cases

      state.reset = false; // Cache the placement in cache to make it available to the modifiers
      // modifiers will modify this one (rather than the one in options)

      state.placement = state.options.placement;
      state.orderedModifiers.forEach(function (modifier) {
        return state.modifiersData[modifier.name] = Object.assign({}, modifier.data);
      });
      var __debug_loops__ = 0;

      for (var index = 0; index < state.orderedModifiers.length; index++) {
        if (process.env.NODE_ENV !== "production") {
          __debug_loops__ += 1;

          if (__debug_loops__ > 100) {
            console.error(INFINITE_LOOP_ERROR);
            break;
          }
        }

        if (state.reset === true) {
          state.reset = false;
          index = -1;
          continue;
        }

        var _state$orderedModifie = state.orderedModifiers[index],
            fn = _state$orderedModifie.fn,
            enabled = _state$orderedModifie.enabled,
            _options = _state$orderedModifie.options;

        if (enabled && typeof fn === 'function') {
          state = fn(state, _options);
        }
      }
    }; // Async and optimistically optimized update
    // it will not be executed if not necessary
    // debounced, so that it only runs at most once-per-tick


    instance.update = microtaskDebounce(function () {
      return new Promise(function (resolve) {
        instance.forceUpdate();
        resolve(state);
      });
    }); // Unwrap `reference` and `popper` elements in case they are
    // wrapped by jQuery, otherwise consume them as is

    var referenceElement = unwrapJqueryElement(reference);
    var popperElement = unwrapJqueryElement(popper);
    state.elements = {
      reference: referenceElement,
      popper: popperElement
    }; // Store options into state

    state.options = Object.assign({}, defaultOptions, {}, options); // Don't proceed if `reference` or `popper` are invalid elements

    if (!areValidElements(referenceElement, popperElement)) {
      if (process.env.NODE_ENV !== "production") {
        console.error(INVALID_ELEMENT_ERROR);
      }

      return;
    }

    state.scrollParents = {
      reference: listScrollParents(referenceElement),
      popper: listScrollParents(popperElement)
    }; // Validate the provided modifiers so that the consumer will get warned
    // of one of the custom modifiers is invalid for any reason

    if (process.env.NODE_ENV !== "production") {
      validateModifiers(state.options.modifiers);
    } // Order `options.modifiers` so that the dependencies are fulfilled
    // once the modifiers are executed


    state.orderedModifiers = orderModifiers([].concat(defaultModifiers, state.options.modifiers)) // Apply user defined preferences to modifiers
    .map(function (modifier) {
      return Object.assign({}, modifier, {}, state.options.modifiers.find(function (_ref) {
        var name = _ref.name;
        return name === modifier.name;
      }));
    }); // Modifiers have the opportunity to execute some arbitrary code before
    // the first update cycle is ran, the order of execution will be the same
    // defined by the modifier dependencies directive.
    // The `onLoad` function may add or alter the options of themselves

    state.orderedModifiers.forEach(function (_ref2) {
      var onLoad = _ref2.onLoad,
          enabled = _ref2.enabled;
      return enabled && onLoad && onLoad(state);
    });

    instance.enableEventListeners = function (eventListeners) {
      var _ref3 = typeof eventListeners === 'boolean' ? expandToHashMap(eventListeners, ['scroll', 'resize']) : eventListeners,
          scroll = _ref3.scroll,
          resize = _ref3.resize;

      if (scroll) {
        var scrollParents = [].concat(state.scrollParents.reference, state.scrollParents.popper);
        scrollParents.forEach(function (scrollParent) {
          return scrollParent.addEventListener('scroll', instance.update, {
            passive: true
          });
        });
      }

      if (resize) {
        var window = getWindow(state.elements.popper);
        window.addEventListener('resize', instance.update, {
          passive: true
        });
      }
    };

    instance.destroy = function () {
      // Remove scroll event listeners
      var scrollParents = [].concat(state.scrollParents.reference, state.scrollParents.popper);
      scrollParents.forEach(function (scrollParent) {
        return scrollParent.removeEventListener('scroll', instance.update);
      }); // Remove resize event listeners

      var window = getWindow(state.elements.popper);
      window.removeEventListener('resize', instance.update); // Run `onDestroy` modifier methods

      state.orderedModifiers.forEach(function (_ref4) {
        var onDestroy = _ref4.onDestroy,
            enabled = _ref4.enabled;
        return enabled && onDestroy && onDestroy(state);
      });
    };

    instance.update().then(function () {
      // After the first update completed, enable the event listeners
      instance.enableEventListeners(state.options.eventListeners);
    });
    return instance;
  };
}
var createPopper = popperGenerator();

export { createPopper, popperGenerator };
//# sourceMappingURL=popper-base.js.map
