// This modifier takes the Popper state and prepares some StyleSheet properties
// that can be applied to the popper element to make it render in the expected position.
export var mapToStyles = function mapToStyles(_ref) {
  var offsets = _ref.offsets,
      position = _ref.position,
      gpuAcceleration = _ref.gpuAcceleration;

  // @1x displays in Chrome can be blurry due to non-rounded offsets, but this
  // introduces slight positioning errors. TODO: somehow solve this better
  // by default it is active, disable it only if explicitly set to false
  if (gpuAcceleration === false || window.devicePixelRatio < 2) {
    return {
      top: offsets.y + "px",
      left: offsets.x + "px",
      position: position
    };
  } else {
    return {
      top: '0px',
      left: '0px',
      transform: "translate3d(" + offsets.x + "px, " + offsets.y + "px, 0)",
      position: position
    };
  }
};
export function computeStyles(state, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$gpuAccelerat = _options.gpuAcceleration,
      gpuAcceleration = _options$gpuAccelerat === void 0 ? true : _options$gpuAccelerat;
  state.modifiersData.computeStyles = {
    styles: {
      // popper offsets are always available
      popper: mapToStyles({
        offsets: state.modifiersData.popperOffsets,
        position: state.options.strategy,
        gpuAcceleration: gpuAcceleration
      }),
      // arrow offsets may not be available
      arrow: state.modifiersData.arrow != null ? mapToStyles({
        offsets: state.modifiersData.arrow,
        position: 'absolute',
        gpuAcceleration: false
      }) : undefined
    },
    attributes: {
      popper: {
        'data-popper-placement': state.placement
      }
    }
  };
  return state;
}
export default {
  name: 'computeStyles',
  enabled: true,
  phase: 'afterMain',
  fn: computeStyles,
  data: {}
};